"""
AgenticAI Framework Tests

Organized test suites:

📁 unit/            - Unit tests for individual components
📁 integration/     - Integration tests for combined features
📁 coverage/        - Coverage-focused tests

Run all tests:
    pytest tests/ -v

Run specific category:
    pytest tests/unit/ -v
    pytest tests/integration/ -v
"""
